// PC Remote Display - eZ2940
// Software is based on the sample "StarterProject"

#include <eZ80.h>
#include <stdio.h>

#include "glcd.h"
#include "hardware.h"

#define VECTOR_UART0  0x70
#define VECTOR_TIMER0 0x54

extern void init_default_vectors(void);
extern void set_vector(unsigned short vector,void (*hndlr)(void));

#define UART                 0
#define UART_BPS             115200
#define UART_DATA_BITS       8
#define UART_PARITY          0
#define UART_STOP_BITS       1

#define DEVICE_NAME          "eZ80F91"

#define UART_RBR             UART0_RBR
#define UART_THR             UART0_THR
#define UART_BRG_L           UART0_BRG_L
#define UART_BRG_H           UART0_BRG_H
#define UART_LCTL            UART0_LCTL
#define UART_LSR             UART0_LSR

#define LCTL_DLAB            (unsigned char)0x80
#define LSR_THRE             (unsigned char)0x20
#define LSR_DR               (unsigned char)0x01

#define SetLCTL(d, p, s)     UART_LCTL = ((d-5)&3) | (((s-1)&1)<<2) | (p&3)

#define LF                   '\n'
#define CR                   '\r'

extern long SysClkFreq;
char device_name[] = DEVICE_NAME;

char NewPicture = 0;
unsigned char PictureBuffer[1024];
int count = 0;
int SerialTimeout = 0;


void uart_init(void) 
{ 
    unsigned short int i;
    unsigned short brg;

    brg = SysClkFreq/(16 * UART_BPS);

    PD_ALT2 = 0x03;
    PD_ALT1 = 0x00;
    PD_DDR = 0xEB;
    PD_DR = 0x00;

    PB_DR = 0x40;
    PB_ALT2 = 0x00;
    PB_ALT1 = 0x00;
    PB_DDR = 0xBF;

    UART_LCTL |= LCTL_DLAB;
    UART_BRG_L = (brg & (unsigned short)0x00FF);
    UART_BRG_H = (brg & (unsigned short)0xFF00) >> (unsigned short)8;
    UART_LCTL &= ~LCTL_DLAB;

    SetLCTL(UART_DATA_BITS, UART_PARITY, UART_STOP_BITS);
} 

void timer_init(void)
{
	TMR0_CTL = TMR0_CTL | B4; // Clock Divide: 256
	TMR0_CTL = TMR0_CTL | B3; // Clock Divide: 256
	TMR0_CTL = TMR0_CTL | B2; // Continous mode
	TMR0_CTL = TMR0_CTL | B1; // Force Auto Reload
	
	TMR0_RR_L = 161;          // Timerinterrupt every 10ms; (50MHz/256)/1953
	TMR0_RR_H = 7;            // Timerinterrupt every 10ms; (50MHz/256)/1953

	TMR0_CTL = TMR0_CTL | B0; // Start Timer
}

int getch(void)
{
    while ((UART_LSR & LSR_DR) == (unsigned char)0);
    return (UART_RBR);
}

int putch(int ch)
{
    while ((UART_LSR & LSR_THRE) == (unsigned char)0);

    UART_THR = ch;

    if (ch == LF)
    {
        while ((UART_LSR & LSR_THRE) == (unsigned char)0);
        UART_THR = CR;
    }

    return (ch);
}

void interrupt UART_Receive(void)
{
	char character;

	character = UART_RBR; // Read received character
 
	SerialTimeout = 0; // Character was received; reset Timeout-Counter

	PictureBuffer[count] = ~character; // Put inverted byte in Picturebuffer
	
	// If picture is complete, set flag for main loop to display it
	count++;
	if (count >= sizeof(PictureBuffer))
	{
    count = 0;
	NewPicture = 1;
	}
}

void interrupt TIMER0_IRQ(void)
{
    char dummy;

	dummy = TMR0_IIR; // Dummy read to clear irq-flag
	
	if (SerialTimeout < 2000)
	{
		SerialTimeout++;
	}
	
	// If no byte was received for 30ms, reset bytecounter
	if (SerialTimeout > 2)
	{
		count = 0;
	}
}

void main(void)
{
    int ch;
    int i = 0;
    const char zds[] = "eZ80F91 Ready";
	unsigned int index;
	unsigned char page;
	unsigned char line;

	init_default_vectors();

    uart_init();
	timer_init();

    printf("\n%s\n", zds);

	// LCD
	GLCD_Init();               // Startup and Init of the Display
	GLCD_SetContrast(B2, 20);
	GLCD_ShowPicture(0);

	// UART Interrupt
    set_vector(VECTOR_UART0, UART_Receive);
	UART0_IER = UART0_IER | B0;

	// TIMER Interrupt
	set_vector(VECTOR_TIMER0, TIMER0_IRQ);
	TMR0_IER = TMR0_IER | B0; // Interrupt @ End of Count

    while (1)
    {
	    // If new picture was received
		if (NewPicture)
		{
			index = 0; // Reset Byteindex

			for (page=0; page<8; page++)
			{
				for (line=0; line <128; line++)
				{
					GLCD_WriteDisplayByte(page, line, PictureBuffer[index]);
					index++;
				}
			}
			NewPicture = 0;
		}

		// If no picture was received for 10 seconds, show standard picture
		if (SerialTimeout > 1000)
		{
			GLCD_ShowPicture(0);
		}
    }
}


